# The goal of this script is to list all the xpm and svg files in this directory and subdirectories
# We will open each file and find each hex color code and replace it with a darker version
# For that we will convert the color to HSV, invert the V component and convert back to RGB
# We will then save the modified file with the name of the original file appended with "Dark"

import os
import re
import math
from subprocess import Popen, PIPE

countries = ['british', 'bulgarian', 'chinese', 'croatian', 'czech', 'danish', 'dutch', 
 'english', 'esperanto', 'finnish', 'french', 'german', 'greek', 'hungarian', 
 'italian', 'japanese', 'korean', 'polish', 'portuguese', 'romanian', 'russian', 
 'slovak', 'slovene', 'spanish', 'swedish', 'taiwanese', 'ukrainian']

def hexToRGB(hex):
    return tuple(int(hex[i:i+2], 16) for i in (0, 2, 4))

def RGBToHex(rgb):
    return '%02x%02x%02x' % rgb

def RGBToHSV(rgb):
    r, g, b = rgb
    r, g, b = r / 255.0, g / 255.0, b / 255.0
    mx = max(r, g, b)
    mn = min(r, g, b)
    df = mx-mn
    if mx == mn:
        h = 0
    elif mx == r:
        h = (60 * ((g-b)/df) + 360) % 360
    elif mx == g:
        h = (60 * ((b-r)/df) + 120) % 360
    elif mx == b:
        h = (60 * ((r-g)/df) + 240) % 360
    if mx == 0:
        s = 0
    else:
        s = (df/mx)*100
    v = mx*100
    return h, s, v

def HSVToRGB(hsv):
    h, s, v = hsv
    h, s, v = h / 360.0, s / 100.0, v / 100.0
    i = int(h*6)
    f = (h*6) - i
    p = v * (1 - s)
    q = v * (1 - f*s)
    t = v * (1 - (1 - f) * s)
    i = i % 6
    if i == 0:
        return v, t, p
    elif i == 1:
        return q, v, p
    elif i == 2:
        return p, v, t
    elif i == 3:
        return p, q, v
    elif i == 4:
        return t, p, v
    elif i == 5:
        return v, p, q

def darkenColor(hex):
    rgb = hexToRGB(hex)
    hsv = RGBToHSV(rgb)
    hsv = (hsv[0], hsv[1], 100 - hsv[2])  # Invert V
    rgb = HSVToRGB(hsv)
    return RGBToHex(tuple(int(x * 255) for x in rgb))

def reverse(rgb):
    r, g, b = rgb
    m = min(r, g, b)
    M = max(r, g, b)
    t = (r + g + b) // 3
    tt = 255 - t
    mu = 1.0

    if M != m:
        lambda1 = max((t - m) / t, (M - t) / (255 - t))
        lambda2 = max((t - m) / tt, (M - t) / (255 - tt))
        mu = lambda1 / lambda2

    r = int(tt + mu * (r - t) + 0.5)
    g = int(tt + mu * (g - t) + 0.5)
    b = int(tt + mu * (b - t) + 0.5)

    return (r, g, b)

import math

# Function to get the maximum value from a list of colors
def get_max(colors, sum_mode=False):
    m = 0
    for color in colors:
        r, g, b = color
        if sum_mode:
            m = max(m, (r + g + b) // 3)  # If sum_mode is True, use the average
        else:
            m = max(m, max(r, g, b))  # Otherwise, use the max of r, g, b
    return m

# Saturation function for a list of RGB tuples
def saturate(colors):
    m = get_max(colors, sum_mode=False)
    s = get_max(colors, sum_mode=True)
    
    # Early exit if max is 0 or s is already high enough
    if m == 0 or s >= 224:
        return colors
    
    # Calculate the scaling factor
    f = min(224.0 / s, 255.0 / m)
    
    # Process each color and apply the saturation
    saturated_colors = []
    for r, g, b in colors:
        r = int(math.floor(f * r + 0.5))
        g = int(math.floor(f * g + 0.5))
        b = int(math.floor(f * b + 0.5))
        saturated_colors.append((r, g, b))
    
    return saturated_colors

def listFonts(file):
    f = open(file, 'r')
    content = f.read()
    fonts = re.findall(r'font-family:([^;]+);', content)
    return fonts

def vectorizeFont(font):
    # check that it is an svg
    if not font.endswith('.svg'):
        return
    # add sufix _vect to the font name
    
    process = Popen(['C:\\Program Files\\Inkscape\\bin\\inkscape.exe', font, '--export-plain-svg=' + font.replace(".svg", "_vect.svg"), '--actions=select-all;object-to-path'], stdout=PIPE, stderr=PIPE)
    
    # wait for the process to finish
    out, err = process.communicate()

    # check if there was an error
    if err:
        print("Error:", err)
        exit(1)

    # check if the file was created
    if not os.path.exists(font.replace(".svg", "_vect.svg")):
        print("Error: file not created", font.replace(".svg", "_vect.svg"))
        exit(1)
    else:
        print("File created", font.replace(".svg", "_vect.svg"))
    

def processFile(file):
    global countries
    if ".dark." in file:
        return
    f = None
    charset = None
    try:
        charset = 'utf-8'
        f = open(file, 'r', encoding='utf-8')
    except UnicodeEncodeError:
        try:
            print("Trying latin1 encoding")
            charset = 'latin1'
            f = open(file, 'r', encoding='latin1')
        except UnicodeEncodeError:
            try:
                print("Trying utf-16 encoding")
                charset = 'utf-16'
                f = open(file, 'r', encoding='utf-16')
            except UnicodeEncodeError:
                print("Error: could not open file", file)
    
    content = f.read()

    #if the file contains "Flag of" it is a flag and we should not darken it
    if "Flag" in content or any(country in content for country in countries) or any(country in file for country in countries) or "TeXmacs" in file:
        print("Flag detected, skipping", file)
        with open(file.replace('.xpm', '.dark.xpm').replace('.svg', '.dark.svg'), 'w', encoding=charset) as f:
            f.write(content)
        return

    # colors = re.findall(r'#[0-9a-fA-F]{6}', content)
    colors = re.findall(r'#[0-9a-fA-F]{6}(?![0-9a-fA-F]{2})', content)
    print(colors)
    rgbColors = [hexToRGB(color[1:]) for color in colors]
    invertedColors = [reverse(color) for color in rgbColors]
    saturatedColors = saturate(invertedColors)
    print(saturatedColors)

    # create a mapping from the original color to the new color
    colorMap = {}
    for i in range(len(colors)):
        colorMap[colors[i]] = '#' + RGBToHex(saturatedColors[i])

    #for i in range(len(colors)):
    #    content = content.replace(colors[i], '#' + RGBToHex(saturatedColors[i]))
    
    # this is not working, because if we replace #ffffff by #000000 and #000000 by #ffffff, we will end up with #ffffff again
    # so we need to use finditer
    for match in re.finditer(r'#[0-9a-fA-F]{6}(?![0-9a-fA-F]{2})', content):
        color = match.group()
        content = content[:match.start()] + colorMap[color] + content[match.end():]


    with open(file.replace('.xpm', '.dark.xpm').replace('.svg', '.dark.svg'), 'w', encoding=charset) as f:
        f.write(content)

def processDirectory(directory):
    fonts = []
    for root, dirs, files in os.walk(directory):
        for file in files:
            if file.endswith('.xpm') or file.endswith('.svg'):
                vectorizeFont(os.path.join(root, file))
                processFile(os.path.join(root, file))
                # fonts += listFonts(os.path.join(root, file))
    # convert the list of fonts to a set to remove duplicates
    fonts = set(fonts)
    print(fonts)

processDirectory('.')